==> 1.4.0.tar.gz: lib/src/mixin_storage.dart <==
part of '../flutterx_live_data.dart';

/// A [LiveData] mixin that stores and reads it's value in from a storage if the value differs from default value
/// Data needs to be serialized and deserialized into a String in order to be stored into a storage map
mixin StorageMixin<T> on LiveData<T> {
  /// Storage key
  String get key;

  /// Default value used when no value with [key] is found in the storage
  T get defaultValue;

  Map<String, String> get _storage;

  String Function(T value) get _encode;

  @override
  void invalidate() {
    if (shouldUpdateValue(defaultValue, _data))
      _storage[key] = _encode(_data);
    else
      _storage.remove(key);
    super.invalidate();
  }

  /// Resets this LiveData to it's [defaultValue]
  void reset() => setData(defaultValue);

  /// Resolve value at [key] from [storage]. If key is missing [defaultValue] is returned, otherwise [decode] will be
  /// used to convert serialized value to an instance of [T]
  static T resolve<T>(String key, Map<String, String> storage, T defaultValue, T Function(String source) decode) {
    final value = storage[key];
    return value == null ? defaultValue : decode(value);
  }

  /// Encode [value] to String. an empty string is returned if value == null
  static String defaultEncode(dynamic value) => value == null ? '' : value.toString();

  /// Decode [source] to String or throw if [T] is not String
  static T defaultDecode<T>(String source) {
    assert(source is T, 'Default decode only supports $String, got $T. Use a custom decode function instead');
    return source as T;
  }

  /// Encode Enum [value] to String
  static String encodeEnum(dynamic value) => value == null ? '' : (value as Enum).name;

  /// Decode [source] to Enum value
  static T decodeEnum<T>(String source, Iterable<T> values) =>
      source.isEmpty ? null as T : (values as Iterable<Enum>).byName(source) as T;

  /// Encode Iterable [value] to json String
  static String encodeJsonIterable(Iterable value) => jsonEncode(value.toList(growable: false));

  /// Decode [source] json to Iterable value
  static Iterable<T> decodeJsonIterable<T>(String source, T Function(dynamic item) map) =>
      (jsonDecode(source) as Iterable).map(map);

  /// Encode Enum Iterable [value] to json String
  static String encodeEnumIterable(Iterable value) => encodeJsonIterable(value.map(encodeEnum));

  /// Decode [source] json to Enum Iterable value
  static Iterable<T> decodeEnumIterable<T>(String source, Iterable<T> values) =>
      decodeJsonIterable(source, (name) => decodeEnum<T>(name, values));
}

==> 1.4.5.tar.gz: lib/src/mixin_storage.dart <==
part of '../flutterx_live_data.dart';

/// A [LiveData] mixin that stores and reads it's value in from a storage if the value differs from default value
/// Data needs to be serialized and deserialized into a String in order to be stored into a storage map
mixin StorageMixin<T> on LiveData<T> {
  /// Storage key
  String get key;

  /// Default value used when no value with [key] is found in the storage
  T get defaultValue;

  /// Map that actually stores [key] value
  @protected
  Map<String, String> get storage;

  /// Encode [T] into a String in order to store it into [storage] map
  @protected
  String Function(T value) get encode;

  @override
  void invalidate() {
    if (shouldUpdateValue(defaultValue, _data))
      storage[key] = encode(_data);
    else
      storage.remove(key);
    super.invalidate();
  }

  /// Resets this LiveData to it's [defaultValue]
  void reset() => setData(defaultValue);

  /// Resolve value at [key] from [storage]. If key is missing [defaultValue] is returned, otherwise [decode] will be
  /// used to convert serialized value to an instance of [T]
  static T resolve<T>(String key, Map<String, String> storage, T defaultValue, T Function(String source) decode) {
    final value = storage[key];
    return value == null ? defaultValue : decode(value);
  }

  /// Encode [value] to String. an empty string is returned if value == null
  static String defaultEncode(dynamic value) => value == null ? '' : value.toString();

  /// Decode [source] to String or throw if [T] is not String
  static T defaultDecode<T>(String source) {
    assert(source is T, 'Default decode only supports $String, got $T. Use a custom decode function instead');
    return source as T;
  }

  /// Encode Enum [value] to String
  static String encodeEnum(dynamic value) => value == null ? '' : (value as Enum).name;

  /// Decode [source] to Enum value
  static T decodeEnum<T>(String source, Iterable<T> values) =>
      source.isEmpty ? null as T : (values as Iterable<Enum>).byName(source) as T;

  /// Encode Iterable [value] to json String
  static String encodeJsonIterable(Iterable value) => jsonEncode(value.toList(growable: false));

  /// Decode [source] json to Iterable value
  static Iterable<T> decodeJsonIterable<T>(String source, T Function(dynamic item) map) =>
      (jsonDecode(source) as Iterable).map(map);

  /// Encode Enum Iterable [value] to json String
  static String encodeEnumIterable(Iterable value) => encodeJsonIterable(value.map(encodeEnum));

  /// Decode [source] json to Enum Iterable value
  static Iterable<T> decodeEnumIterable<T>(String source, Iterable<T> values) =>
      decodeJsonIterable(source, (name) => decodeEnum<T>(name, values));
}
